@@ -7,7 +7,7 @@ use JSON;
 use Colouring::In;
 use Image::Scale;
 use Cwd qw/abs_path/;
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 our (%TOOL, %MANIFEST_SCHEMA);
 BEGIN {
 	%TOOL = (
@@ -31,8 +31,8 @@ BEGIN {
 				$path .= "/$_";
 				$path =~ $TOOL{tainted};
 				if (! -d $1) {
-					mkdir $1 or Carp::craok(qq/
-						Cannot create path $1 $!
+					mkdir $1  or Carp::croak(qq/
+						Cannot open file for writing $!
 					/);
 				}
 			}
@@ -61,7 +61,8 @@ BEGIN {
 				Cannot open file for reading $!
 			/);
 			my $content = do { local $/; <$fh> };
-			close $fh && chomp($content);
+			close $fh;
+			chomp($content);
 			return $content;
 		},
 		remove_file => sub { $TOOL{remove_directory}->($TOOL{abs}->($_[0])); },
@@ -69,7 +70,7 @@ BEGIN {
 			my ($val, %opts) = $TOOL{parse_params}->(@_);
 			$val = $TOOL{abs}->($val);
 			$val =~ $TOOL{tainted};
-			opendir(my $DIR, $1) or die "Cant open $1$!";
+			opendir(my $DIR, $1) or die "Can't open $1$!";
 			my @files;
 			for my $file (grep {$_ !~ /^\./} readdir($DIR)) {
 				my $path = sprintf "%s/%s", $val, $file;
@@ -78,7 +79,7 @@ BEGIN {
 				push @files, $opts{recurse} && -d $path
 					?  map {
 						sprintf "%s/%s", $file, $_;
-					} $TOOL{read_directory}->($path)
+					} $TOOL{read_directory}->($path, %opts)
 					: $path;
 			}
 			closedir ($DIR) or die "Cant close $1$!";
@@ -97,7 +98,7 @@ BEGIN {
 				rmdir($d);
 			} else {
 				unlink $1 or Carp::croak(qq/
-					cannot remove file $1$!
+					Cannot remove file $1$!
 				/);
 			}
 			return 1;
@@ -113,7 +114,7 @@ BEGIN {
 		generate_icons => sub {
 			my ($icon, %options) = $TOOL{parse_params}->(@_);
 			Carp::croak('No initial icon passed to generate_icons') if !$icon;
-			my $img = Image::Scale->new($icon) || Carp::croak "Invalid file";
+			my $img = Image::Scale->new($icon);
 			$options{outpath} = substr($icon, 0, rindex($icon, '/')) if !$options{outpath};
 			$options{outpath} =~ s/\/$//;
 			$TOOL{make_path}->($options{outpath});
@@ -149,7 +150,7 @@ BEGIN {
 			return @files;
 		},
 		identify_icon_size => sub {
-			my $img = Image::Scale->new($TOOL{abs}->($_[0])) || Carp::croak "Invalid file";
+			my $img = Image::Scale->new($TOOL{abs}->($_[0]));
 			my $size = sprintf "%sx%s", $img->width(), $img->height();
 			return $TOOL{valid_icon_sizes}->{$size} ? $size : undef;;
 		},
@@ -251,12 +252,13 @@ BEGIN {
 			$TOOL{array_check}->(@_);
 			for my $app (@{$_[0]}) {
 				Carp::croak(q|related_applicaiton is not a HASH|) unless ref $app eq q|HASH|;
-				$app->{$_} or Carp::croak(sprintf
-					q|Missing required param %s in related_application %s|,
-					$_,
-					$TOOL{to_json}->($app)
-				)
-				for (qw/platform url/, ($app->{platform} || "") =~ m/play/i ? 'id' : ());
+				for (qw/platform url/, ($app->{platform} || "") =~ m/play/i ? 'id' : ()) {
+					Carp::croak(sprintf
+						q|Missing required param %s in related_application %s|,
+						$_,
+						$TOOL{to_json}->($app)
+					) unless $app->{$_};
+				}
 			}
 			return $_[0];
 		},
@@ -266,12 +268,13 @@ BEGIN {
 			$TOOL{array_check}->(@_);
 			for my $screenshot (@{$_[0]}) {
 				Carp::croak(q|screenshot is not a HASH|) unless ref $screenshot eq q|HASH|;
-				$screenshot->{$_} or Carp::croak(sprintf
-					q|Missing required param %s in screenshot %s|,
-					$_,
-					$TOOL{to_json}->($screenshot)
-				)
-				for (qw/src sizes type/);
+				map { 
+					Carp::croak(sprintf
+						q|Missing required param %s in screenshot %s|,
+						$_,
+						$TOOL{to_json}->($screenshot)
+					) unless $screenshot->{$_};
+				} qw/src sizes type/;
 			}
 			# TODO - once known to be fully supported...
 			# Mechanize->screenshot
@@ -322,7 +325,7 @@ BEGIN {
 sub new {
 	my ($package, %args) = $TOOL{parse_params}->(@_);
 	my $new = bless {}, $package;
-	exists $args{$_} && do {$new->{$_} = $args{$_}} for qw/root pathpart/;
+	for (qw/root pathpart/) { $new->{$_} = $args{$_} if exists $args{$_}; }
 	$new->set_manifest($args{manifest}) if $args{manifest};
 	$new->set_params($args{params}) if $args{params};
 	$new->set_template(%args);
@@ -351,14 +354,15 @@ sub set_manifest {
 	my ($self, %args) = $TOOL{parse_params}->(@_);
 	for my $key (keys %args) {
 		my $validate = $MANIFEST_SCHEMA{$key}
-			or Carp::croak(sprintf q/Invalid key passed to setup manifest %s/, $key)
-				and return;
+			or Carp::croak(sprintf q/Invalid key passed to setup manifest %s/, $key);
 		my $val = $validate->($args{$key}, $key, $self->{root}, $self->{pathpart});
 		$self->{manifest}{$key} = $val;
 	}
 	$self->{manifest};
 }
 
+sub clear_manifest { delete $_[0]->{manifest}; }
+
 sub has_params { $_[0]->{params} ? 1 : 0 }
 
 sub params { $_[1] ? $TOOL{to_json}->($_[0]->{params}) : $_[0]->{params}; }
@@ -390,13 +394,16 @@ sub set_params {
 	$self->{params};
 }
 
+sub clear_params { delete $_[0]->{params}; }
+
 sub template { $_[0]->{template} }
 
 sub templates { $_[0]->{template}->render($_[0]->{params}); }
 
 sub compile {
 	my ($self, $root) = @_;
-	$root = $self->{root} || Carp::croak(
+	$root ||= $self->{root};
+	Carp::croak(
 		q/No root directory provided to compile manifest and service worker/
 	) unless $root;
 	$TOOL{make_path}->($root);
@@ -405,7 +412,7 @@ sub compile {
 		($self->has_params ? (templates => $self->templates()) : ()),
 	);
 	$TOOL{write_file}->(sprintf("%s/manifest.json", $root), $build{manifest}) if $build{manifest};
-	if ($build{templates} && keys %{$self->{params}} > 1) {
+	if ($build{templates}) {
 		for my $template (keys %{$build{templates}}) {
 			$TOOL{write_file}->(
 	 			sprintf("%s/%s", $root, $template),
@@ -427,7 +434,7 @@ Progressive::Web::Application - Utility for making an application 'progressive'
 
 =head1 VERSION
 
-Version 0.04
+Version 0.05
 
 =cut
 
@@ -698,6 +705,14 @@ Set the manifest params see manifest_schema for full options.
 
 =cut
 
+=head2 clear_manifest
+
+Clear/Delete manifest from the object, this is usefull for when you call 'compile' to different outpaths multiple times within a script.
+
+	$pwa->clear_manifest();
+
+=cut
+
 =head2 has_params
 
 Does the Progressive::Web::Application have params configured, returns true or false value (1|0).
@@ -732,6 +747,14 @@ Set the template params see the configured template for what is required.
 
 =cut
 
+=head2 clear_params
+
+Clear/Delete params from the object, this is usefull for when you call 'compile' to different outpaths multiple times within a script.
+
+	$pwa->clear_params();
+
+=cut
+
 =head2 template
 
 Returns the template object.
